package org.twinone.ejempljson.sqlite;

import java.util.ArrayList;
import java.util.List;

import org.twinone.ejempljson.Amigo;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

public class AyudanteBD extends SQLiteOpenHelper {

	// Nombre de la base de datos
	private static final String NOMBRE_BD = "ejemplo";

	// Nombre de la tabla
	private static final String NOMBRE_TABLA = "amigos";

	// Nombres de las columnas (se corresponden con las claves en JSON)
	private static final String COL_ID = "id";
	private static final String COL_NOMBRE = "nombre";
	private static final String COL_EDAD = "edad";
	private static final String COL_TRABAJO = "trabajo";
	private static final String COL_DEPORTE = "deporte";
	private static final String COL_CASADO = "casado";
	private static final String COL_IMAGEN = "imagen";

	// CREATE TABLE amigos (id INTEGER, nombre TEXT(30), edad INTEGER...)
	private static final String CONSULTA_CREAR_TABLA =

	"CREATE TABLE " + NOMBRE_TABLA + "(" +

	COL_ID + " INTEGER PRIMARY KEY," +

	COL_NOMBRE + " TEXT(30)," +

	COL_EDAD + " INTEGER," +

	COL_TRABAJO + " TEXT(30)," +

	COL_DEPORTE + " TEXT(30)," +

	COL_IMAGEN + " TEXT(2048)," +

	COL_CASADO + " INTEGER" +

	")";

	// función interna para obtener un objeto Amigo a partir de un Cursor
	private Amigo obtenerAmigoDeCursor(Cursor cursor) {
		Amigo amigo = new Amigo();
		amigo.Nombre = cursor.getString(cursor.getColumnIndex(COL_NOMBRE));
		amigo.Edad = cursor.getInt(cursor.getColumnIndex(COL_EDAD));
		amigo.Trabajo = cursor.getString(cursor.getColumnIndex(COL_TRABAJO));
		amigo.Deporte = cursor.getString(cursor.getColumnIndex(COL_DEPORTE));
		// Como no existen booleanos en SQLite, usamos 0=false, 1=true
		amigo.Casado = cursor.getInt(cursor.getColumnIndex(COL_CASADO)) == 1;

		amigo.Imagen = cursor.getString(cursor.getColumnIndex(COL_IMAGEN));

		return amigo;
	}

	// función interna para introducir un amigo la base de datos
	private ContentValues crearValoresAPartirDeAmigo(Amigo amigo) {
		ContentValues valores = new ContentValues();
		valores.put(COL_NOMBRE, amigo.Nombre);
		valores.put(COL_EDAD, amigo.Edad);
		valores.put(COL_TRABAJO, amigo.Trabajo);
		valores.put(COL_DEPORTE, amigo.Deporte);
		// Como no existe booleanos en SQLite, usamos 0=false, 1=true
		valores.put(COL_CASADO, amigo.Casado ? 1 : 0);

		valores.put(COL_IMAGEN, amigo.Imagen);
		return valores;
	}

	/**
	 * Añade un amigo a la base de datos y devuelve su ID
	 */
	public long insertarAmigo(Amigo amigo) {

		// Obtenemos una base de datos en modo solo escritura
		SQLiteDatabase db = getWritableDatabase();
		ContentValues valores = crearValoresAPartirDeAmigo(amigo);

		// insert() nos da la del amigo que hemos insertado
		long id = db.insert(NOMBRE_TABLA, null, valores);
		return id;
	}

	/**
	 * Devuelve un unico amigo con la ID especificada. Puede ser nulo si el
	 * amigo no existe
	 * 
	 * @param id
	 * @return
	 */
	public Amigo seleccionarAmigo(long id) {
		// Obtenemos la base de datos en modo solo lectura
		SQLiteDatabase db = getReadableDatabase();

		String consulta = "SELECT * FROM " + NOMBRE_TABLA + " WHERE " + COL_ID
				+ " = " + id;
		// Obtenemos un cursor del que podemos leer un amigo
		Cursor cursor = db.rawQuery(consulta, null);

		// Si el cursor es nulo o no hay nada en la primera posicion, devolvemos
		// un Amigo nulo.
		if (cursor == null || !cursor.moveToFirst()) {
			return null;
		}

		Amigo amigo = obtenerAmigoDeCursor(cursor);
		cursor.close();
		return amigo;
	}

	/**
	 * Igual que getAmigoId, pero con el nombre.
	 * 
	 * @param nombre
	 * @return
	 */
	public Amigo seleccionarAmigo(String nombre) {
		SQLiteDatabase db = getReadableDatabase();

		String consulta = "SELECT * FROM " + NOMBRE_TABLA + " WHERE "
				+ COL_NOMBRE + " = " + nombre;
		Cursor cursor = db.rawQuery(consulta, null);

		if (cursor == null || !cursor.moveToFirst()) {
			return null;
		}

		Amigo amigo = obtenerAmigoDeCursor(cursor);
		cursor.close();
		return amigo;
	}

	/**
	 * Lista de Amigos.<br>
	 * Ni la lista ni los amigos dentro de ella pueden ser nulos, pero la lista
	 * puede estar vacia
	 * 
	 * @return
	 */
	public List<Amigo> seleccionarTodosLosAmigos() {
		List<Amigo> amigos = new ArrayList<Amigo>();
		String consulta = "SELECT  * FROM " + NOMBRE_TABLA;

		SQLiteDatabase db = this.getReadableDatabase();
		Cursor cursor = db.rawQuery(consulta, null);

		// Si el cursor es nulo o no hay nada en la primera posicion, devolvemos
		// una lista vacia.
		if (cursor == null || !cursor.moveToFirst()) {
			return amigos;
		}

		do {
			Amigo amigo = obtenerAmigoDeCursor(cursor);
			amigos.add(amigo);
		} while (cursor.moveToNext());

		return amigos;
	}

	public int contarAmigos() {
		SQLiteDatabase db = this.getReadableDatabase();

		String consulta = "SELECT * FROM " + NOMBRE_TABLA;

		Cursor cursor = db.rawQuery(consulta, null);

		int count = cursor.getCount();
		cursor.close();

		// return count
		return count;
	}

	/**
	 * Borrar un amigo por ID
	 */
	public void borrarAmigo(long id) {
		SQLiteDatabase db = getWritableDatabase();
		String[] ids = new String[] { String.valueOf(id) };
		db.delete(NOMBRE_TABLA, COL_ID + " = ?", ids);
	}

	/**
	 * Borrar un amigo por Nombre
	 */
	public void borrarAmigo(String nombre) {
		SQLiteDatabase db = getWritableDatabase();
		String[] ids = new String[] { String.valueOf(nombre) };
		db.delete(NOMBRE_TABLA, COL_NOMBRE + " = ?", ids);
	}

	public void borrarTodosLosAmigos() {
		SQLiteDatabase db = getWritableDatabase();
		String consulta = "DELETE FROM " + NOMBRE_TABLA;
		db.execSQL(consulta);
	}

	/**
	 * Ejecuta una consulta y devuelve true si se ha ejecutado correctamente.
	 * 
	 * @return
	 */
	public boolean ejecutarSQL(String sql) {
		try {
			getWritableDatabase().execSQL(sql);
		} catch (Exception e) {
			return false;
		}
		return true;
	}

	/**
	 * 
	 * 
	 * 
	 * 
	 * Cosas internas
	 * 
	 * 
	 * 
	 * 
	 */

	public AyudanteBD(Context context) {
		// Le decimos a Android que haga la base de datos con el nombre
		// NOMBRE_BD, y versión 1
		super(context, NOMBRE_BD, null, 1);
	}

	@Override
	public void onCreate(SQLiteDatabase db) {
		// Cuando se crea la base de datos creamos la tabla
		Log.d("AyudanteBD", CONSULTA_CREAR_TABLA);
		db.execSQL(CONSULTA_CREAR_TABLA);
	}

	// Qué pasa cuando se actualiza la base de datos
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {

		onCreate(db);
	}

	/**
	 * Llamar closeDB cuando ya no la usamos
	 */
	public void closeDB() {
		SQLiteDatabase db = this.getReadableDatabase();
		if (db != null && db.isOpen())
			db.close();
	}
}